#=======================================================================
#	$Id: Obj.pm,v 1.2 2006/01/18 21:05:23 pythontech Exp $
#	Base class for object (resource) mapped to a triples model
#	Copyright (C) 2004  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::Obj;
use strict;

sub new {
    my($class, $uri, $model) = @_;
    my $self = {
	uri => $uri,
	model => $model,
    };
    bless $self, $class;
    return $self;
}

#-----------------------------------------------------------------------
#	Clean access to internals
#-----------------------------------------------------------------------
sub uri {
    my($self) = @_;
    return $self->{'uri'};
}

sub model {
    my($self) = @_;
    return $self->{'model'};
}

#-----------------------------------------------------------------------
#	Queries
#-----------------------------------------------------------------------
sub test {
    my($self, $pred,$obj) = @_;
    return $self->{'model'}->test($self,$pred,$obj);
}

sub has_type {
    my($self,$type) = @_;
    return $self->{'model'}->test($self,'rdf:type',\$type);
}

sub attr {
    my($self, $pred) = @_;
#    print STDERR "attr($pred)\n";
    $self->{'model'}->sp2o($self, $pred);
}

sub rattr {
    my($self, $pred) = @_;
#    print STDERR "rattr($pred)\n";
    $self->{'model'}->po2s($pred, $self);
}

#-----------------------------------------------------------------------
#	Modifications
#-----------------------------------------------------------------------
sub add {
    my($self, $pred,$obj) = @_;
    $self->{'model'}->add($self, $pred, $obj);
}

#-----------------------------------------------------------------------
#	Acquisition interface for PythonTech::Template
#-----------------------------------------------------------------------
sub GET {
    my($self, $var) = @_;
    if ($var eq 'uri') {
	return $self->{'uri'};
    } elsif (my($attr) = $var =~ /^p_(\w+)$/) {
	# Single-valued property
	my $value = $self->attr($attr);
	return $value;
    } elsif (my($attr) = $var =~ /^r_(\w+)$/) {
	# Single-valued reverse relationship
	my $value = $self->rattr($attr);
	return $value;
    } elsif (my($rev,$attr) = $var =~ /^(r?)a_(\w+)$/) {
	# Multiple-valued relationship, optionally reversed
	my @array = $rev ? $self->rattr($attr) : $self->attr($attr);
	return \@array;
    } elsif (my($attr) = $var =~ /^e_(\w+)$/) {
	# Enumerated type - attribute whose value is subclass
	my $enum = $self->attr($attr);
	if ($enum) {
	    my $uri = $enum->uri; # E.g. 'ds:VersionCreated'
	    $uri =~ s!.*\W!!;	# E.g. 'VersionCreated'
	    return $uri;
	} else {
	    return '?';
	}
#    } elsif (my($attr,$enum) = 
#	     $var =~ /^([a-z]+[A-Z][a-zA-Z]+)_([A-Z][a-zA-Z0-9]+)$/) {
#	# Test for enumerated value
#	return $self->test("ds:$attr", \ "ds:$enum") ? 1 : 0;
    } elsif (my($type) = $var =~ /^isa_(\w+)$/) {
	return $self->test('rdf:type', \ $type);
    } else {
	return undef;
    }
}

1;
