# $Id: 01_queries.t,v 1.2 2006/01/18 22:39:38 pythontech Exp pythontech $
use strict;
use Test::More tests => 19;

# FIXME
BEGIN {
    # Need Triples & QMap modules but want local Model & QMap
    # Put site libs at end of search path
    push @INC, '/home/pythontech/lib';
}
use PythonTech::Triples::Memory;
use PythonTech::QMap;
use PythonTech::Model;

# Create test data
my $tr = PythonTech::Triples::Memory->new;
my @data = qw(
	    x:foo rdf:type xs:Thing
	    x:foo xs:name =Fooey
	    x:bar rdf:type xs:Stuff
	    x:bar xs:owner x:foo
);
while (my($s,$p,$o) = splice(@data,0,3)) {
    $tr->add($s,$p,$o);
}

# Tests with no URI manipulation
my $model = PythonTech::Model->new(triples => $tr,
				   classmap => {},
				   dbqmap => undef,
				   extqmap => undef);

my $foo = $model->as_obj('x:foo');
is(ref($foo), 'PythonTech::Obj', 'found obj');
is($foo && $foo->uri, 'x:foo', 'check URI');
is($foo && $foo->attr('xs:name'), 'Fooey', 'literal attribute');
my $bar = $foo && $foo->rattr('xs:owner');
is(ref($bar), 'PythonTech::Obj', 'reverse attr');
is($bar && $bar->uri, 'x:bar', 'check URI');
is($bar && $bar->attr('xs:owner'), $foo, 'check cache');

# Tests with map on triple database
my $dbq = PythonTech::QMap->new(qmap => {
    x => 'ex:obj/',
    xs => 'ex:schema#',
});
my $model2 = PythonTech::Model->new(triples => $tr,
				    classmap => {},
				    dbqmap => $dbq,
				    extqmap => undef);

my $foo2 = $model2->as_obj('ex:obj/foo');
is(ref($foo2), 'PythonTech::Obj', 'found obj');
is($foo2 && $foo2->uri, 'ex:obj/foo', 'check URI');
is($foo2 && $foo2->attr('ex:schema#name'), 'Fooey', 'literal attribute');
my $bar2 = $foo2 && $foo2->rattr('ex:schema#owner');
is(ref($bar2), 'PythonTech::Obj', 'reverse attr');
is($bar2 && $bar2->uri, 'ex:obj/bar', 'check URI');
is($bar2 && $bar2->attr('xs:owner'), $foo2, 'check cache');

# Tests with map on external view
my $extq = PythonTech::QMap->new(qmap => {
    y => 'x:',
    ys => 'xs:',
});
my $model3 = PythonTech::Model->new(triples => $tr,
				    classmap => {},
				    dbqmap => undef,
				    extqmap => $extq);

my $foo3 = $model3->as_obj('y:foo');
is(ref($foo3), 'PythonTech::Obj', 'found obj');
is($foo3 && $foo3->uri, 'y:foo', 'check URI');
is($foo3 && $foo3->attr('ys:name'), 'Fooey', 'literal attribute');
is($foo3 && $foo3->attr('xs:name'), 'Fooey', 'literal attribute, map bypassed');
my $bar3 = $foo3 && $foo3->rattr('ys:owner');
is(ref($bar3), 'PythonTech::Obj', 'reverse attr');
is($bar3 && $bar3->uri, 'y:bar', 'check URI');
is($bar3 && $bar3->attr('ys:owner'), $foo3, 'check cache');
